################################################################
#                         WRAPPER                              #
################################################################
##### Description
# This function calls the model over parameters specified and
# extracts the value of a given varialbe (want) for the last
# timestep. It defaults to extracting equilibrium biomass.# 
# 
################################################################

####################################################################
####    SCENARIO 0 - FISHERY SHORT TERM PROFIT MAXIMIZATION     #### 
####################################################################

wrapper0 <- function(chi, r, K, X0, s, D, p, q, c, beta, L, alpha, mu, w, years, want = "X_vec", b = 0){
  
  if(!want == "All"){
    value <- model0(chi = chi,
                   r = r,
                   K = K,
                   X0 = X0,
                   s = s,
                   D = D,
                   p = p,
                   q = q,
                   c = c,
                   beta = beta,
                   L = L,
                   alpha = alpha,
                   mu = mu,
                   w = w,
                   years = years,
                   tolerance = 0.05,
                   b = b) %>%     # run the model
      filter(time == max(time)) %>%                                          # keep the last timestep only
      pull({{want}})                                                         # return desired variable 
  } else {
    value <- model0(chi = chi,
                   r = r,
                   K = K,
                   X0 = X0,
                   s = s,
                   D = D,
                   p = p,
                   q = q,
                   c = c,
                   beta = beta,
                   L = L,
                   alpha = alpha,
                   mu = mu,
                   w = w,
                   years = years,
                   tolerance = 0.05,
                   b = b) %>%     # run the model
      filter(time == max(time))                                              # keep the last timestep only
  }
  
  return(value)                                                            # Return the value
}


###############################################
####    SCENARIO 1 - CONTROL (OA + MEY)    #### 
###############################################

wrapper1 <- function(chi, r, K, X0, s, D, p, q, c, beta, L, alpha, mu, w, years, want = "X_vec", b = 0){
  
  if(!want == "All"){
    value <- model1(chi = chi,
                   r = r,
                   K = K,
                   X0 = X0,
                   s = s,
                   D = D,
                   p = p,
                   q = q,
                   c = c,
                   beta = beta,
                   L = L,
                   alpha = alpha,
                   mu = mu,
                   w = w,
                   years = years,
                   tolerance = 0.05,
                   b = b) %>%     # run the model
      filter(time == max(time)) %>%                                          # keep the last timestep only
      pull({{want}})                                                         # return desired variable 
  } else {
    value <- model1(chi = chi,
                   r = r,
                   K = K,
                   X0 = X0,
                   s = s,
                   D = D,
                   p = p,
                   q = q,
                   c = c,
                   beta = beta,
                   L = L,
                   alpha = alpha,
                   mu = mu,
                   w = w,
                   years = years,
                   tolerance = 0.05,
                   b = b) %>%     # run the model
      filter(time == max(time))                                              # keep the last timestep only
  }
  
  return(value)                                                            # Return the value
}


########################################
####    SCENARIO 2 - FISH PRICE     #### 
########################################

wrapper2 <- function(chi, r, K, X0, s, D, p, pL, q, c, beta, L, alpha, mu, w, years, want = "X_vec", b = 0){
  
  if(!want == "All"){
    value <- model2(chi = chi,
                   r = r,
                   K = K,
                   X0 = X0,
                   s = s,
                   D = D,
                   p = p,
                   pL = pL,
                   q = q,
                   c = c,
                   beta = beta,
                   L = L,
                   alpha = alpha,
                   mu = mu,
                   w = w,
                   years = years,
                   tolerance = 0.05,
                   b = b) %>%     # run the model
      filter(time == max(time)) %>%                                          # keep the last timestep only
      pull({{want}})                                                         # return desired variable 
  } else {
    value <- model2(chi = chi,
                   r = r,
                   K = K,
                   X0 = X0,
                   s = s,
                   D = D,
                   p = p,
                   pL = pL,
                   q = q,
                   c = c,
                   beta = beta,
                   L = L,
                   alpha = alpha,
                   mu = mu,
                   w = w,
                   years = years,
                   tolerance = 0.05,
                   b = b) %>%     # run the model
      filter(time == max(time))                                              # keep the last timestep only
  }
  
  return(value)                                                            # Return the value
}


##############################################
####    SCENARIO 3  -  DETECTION SPEED    #### 
##############################################

wrapper3 <- function(chi, r, K, X0, s, D, p, q, c, beta, L, alpha, muL, muFPA,  w, years, want = "X_vec", b = 0){
  
  if(!want == "All"){
    value <- model3(chi = chi,
                   r = r,
                   K = K,
                   X0 = X0,
                   s = s,
                   D = D,
                   p = p,
                   q = q,
                   c = c,
                   beta = beta,
                   L = L,
                   alpha = alpha,
                   muL = muL,
                   muFPA = muFPA,
                   w = w,
                   years = years,
                   tolerance = 0.05,
                   b = b) %>%     # run the model
      filter(time == max(time)) %>%                                          # keep the last timestep only
      pull({{want}})                                                         # return desired variable 
  } else {
    value <- model3(chi = chi,
                   r = r,
                   K = K,
                   X0 = X0,
                   s = s,
                   D = D,
                   p = p,
                   q = q,
                   c = c,
                   beta = beta,
                   L = L,
                   alpha = alpha,
                   muL = muL,
                   muFPA = muFPA,
                   w = w,
                   years = years,
                   tolerance = 0.05,
                   b = b) %>%     # run the model
      filter(time == max(time))                                              # keep the last timestep only
  }
  
  return(value)                                                            # Return the value
}

##########################################
####    SCENARIO 4  -  ACCESS FEE     #### 
##########################################

wrapper4 <- function(chi, r, K, X0, s, D, p, q, c, beta, L, alpha, mu, w, years, want = "X_vec", b = 0){
  
  if(!want == "All"){
    value <- model4(chi = chi,
                   r = r,
                   K = K,
                   X0 = X0,
                   s = s,
                   D = D,
                   p = p,
                   q = q,
                   c = c,
                   beta = beta,
                   L = L,
                   alpha = alpha,
                   mu = mu,
                   w = w,
                   years = years,
                   tolerance = 0.05,
                   b = b) %>%     # run the model
      filter(time == max(time)) %>%                                          # keep the last timestep only
      pull({{want}})                                                         # return desired variable 
  } else {
    value <- model4(chi = chi,
                   r = r,
                   K = K,
                   X0 = X0,
                   s = s,
                   D = D,
                   p = p,
                   q = q,
                   c = c,
                   beta = beta,
                   L = L,
                   alpha = alpha,
                   mu = mu,
                   w = w,
                   years = years,
                   tolerance = 0.05,
                   b = b) %>%     # run the model
      filter(time == max(time))                                              # keep the last timestep only
  }
  
  return(value)                                                            # Return the value
}


#################################################
####    SCENARIO 5  -  COMBINED SCENARIOS    #### 
#################################################

wrapper5 <- function(chi, r, K, X0, s, D, p, pL, q, c, beta, L, alpha, muL, muFPA,  w, years, want = "X_vec", b = 0){
  
  if(!want == "All"){
    value <- model5(chi = chi,
                   r = r,
                   K = K,
                   X0 = X0,
                   s = s,
                   D = D,
                   p = p,
                   pL = pL, 
                   q = q,
                   c = c,
                   beta = beta,
                   L = L,
                   alpha = alpha,
                   muL = muL,
                   muFPA = muFPA,
                   w = w,
                   years = years,
                   tolerance = 0.05,
                   b = b) %>%     # run the model
      filter(time == max(time)) %>%                                          # keep the last timestep only
      pull({{want}})                                                         # return desired variable 
  } else {
    value <- model5(chi = chi,
                   r = r,
                   K = K,
                   X0 = X0,
                   s = s,
                   D = D,
                   p = p,
                   pL = pL,
                   q = q,
                   c = c,
                   beta = beta,
                   L = L,
                   alpha = alpha,
                   muL = muL,
                   muFPA = muFPA,
                   w = w,
                   years = years,
                   tolerance = 0.05,
                   b = b) %>%     # run the model
      filter(time == max(time))                                              # keep the last timestep only
  }
  
  return(value)                                                            # Return the value
}

